// handlers.cpp

#include "handlers.h"
#include "stdio.h"

// Externs
extern LARGE_INTEGER	gStartTime;
extern LARGE_INTEGER	gEndTime;
extern DWORD			gStartTick;
extern DWORD			gEndTick;
extern unsigned int		gTotalExecutions;
extern unsigned int		gTestCount;
extern cMutex			gTestMutex;
extern ep_t				myEndpoint;

// Return to sender handler.
void ReturnToSender(int status, op_t opcode, argblock_t argblock)
{
	printf("[RTS] The return to sender handler has run\n");
	printf("Args: a0=%d, a1=%d, a2=%d, a3=%d\n", argblock->args[0], argblock->args[1], argblock->args[2], argblock->args[3]);
	printf("Opcode: %d\n", opcode);
	printf("Reason: %d\n", status);
	fflush(stdout);
}

// Simple round trip request-reply w/ no data.
void RoundTripRequest(void *token, int arg0, int arg1, int arg2, int arg3)
{
	int retVal;
	retVal = AM_Reply4(token, ROUND_TRIP_REPLY, 4, 3, 2, 1);
	if(retVal != AM_OK)
	{
		AM_Error("Unable to reply from RoundTripRequest handler", __LINE__, __FILE__, retVal);
	}
}

// Simple round trip request-reply w/ no data.
void RoundTripReply(void *token, int arg0, int arg1, int arg2, int arg3)
{
	int retVal;

	if(gTestCount < gTotalExecutions)
	{
		gTestCount++;
		retVal = AM_Request4(myEndpoint, 2, ROUND_TRIP_REQUEST, 5, 2, 1, 0);
		if(retVal != AM_OK)
		{
			AM_Error("Unable to send active message in RoundTripReply.", __LINE__, __FILE__, retVal);
		}
	}
	else
	{
		QueryPerformanceCounter(&gEndTime);
		gEndTick = GetTickCount();
		gTestMutex.Unlock();
	}
	
}

// Simple round trip request-reply w/ data.
void RoundTripRequestI(void *token, void *buffer, int nbytes, int arg0, int arg1, int arg2, int arg3)
{
	int retVal;
	retVal = AM_ReplyI4(token, ROUND_TRIP_REPLYI, buffer, nbytes, 4, 3, 2, 1);
	if(retVal != AM_OK)
	{
		AM_Error("Unable to reply from RoundTripRequestI handler", __LINE__, __FILE__, retVal);
	}
}

// Simple round trip request-reply w/ data.
void RoundTripReplyI(void *token, void *buffer, int nbytes, int arg0, int arg1, int arg2, int arg3)
{
	int retVal;

	if(gTestCount < gTotalExecutions)
	{
		gTestCount++;
		retVal = AM_RequestI4(myEndpoint, 2, ROUND_TRIP_REQUESTI, buffer, nbytes, 5, 2, 1, 0);
		if(retVal != AM_OK)
		{
			AM_Error("Unable to send active message in RoundTripReplyI.", __LINE__, __FILE__, retVal);
		}
	}
	else
	{
		QueryPerformanceCounter(&gEndTime);
		gEndTick = GetTickCount();
		gTestMutex.Unlock();
	}
}

// Simple round trip request-reply w/ data.
void RoundTripRequestXfer(void *token, void *buffer, int nbytes, int arg0, int arg1, int arg2, int arg3)
{
	int retVal;
	retVal = AM_ReplyXfer4(token, 0, ROUND_TRIP_REPLYXFER, buffer, 0, nbytes, 3, 2, 1);
	if(retVal != AM_OK)
	{
		AM_Error("Unable to reply from RoundTripRequestXfer handler", __LINE__, __FILE__, retVal);
	}
}

// Simple round trip request-reply w/ data.
void RoundTripReplyXfer(void *token, void *buffer, int nbytes, int arg0, int arg1, int arg2, int arg3)
{
	int retVal;

	if(gTestCount < gTotalExecutions)
	{
		gTestCount++;
		retVal = AM_RequestXfer4(myEndpoint, 2, 0, ROUND_TRIP_REQUESTXFER, buffer, arg0, 3, 2, 1, 0);
		if(retVal != AM_OK)
		{
			AM_Error("Unable to send active message in RoundTripReplyI.", __LINE__, __FILE__, retVal);
		}
	}
	else
	{
		QueryPerformanceCounter(&gEndTime);
		gEndTick = GetTickCount();
		gTestMutex.Unlock();
	}
}