#include <windows.h>

#include <stdio.h>
#include <stdlib.h>
#include <vi/stdvi.h>
#include <am.h>
#include <cMutex.h>

#include "handlers.h"

void AM_SetErrorStream(ostream &errStream);

#define GIGANET_DEVICE_NAME	"nic0"

void RoundTripTest(char **argv, bool server);
void RoundTripTestI(char **argv, bool server);
void VIRoundTripTest(int argc, char **argv, bool server);
void RoundTripTestXfer(char **argv, bool server);

//volatile bool reply_recvd = false;
cMutex	mutex1, mutex2, mutex3;
cMutex	gTestMutex;
int		h1 = 0;
int		h2 = 0;
int		h3 = 0;
int		rep = 0;

DWORD	gStartTick;
DWORD	gEndTick;
	
LARGE_INTEGER	 gFrequency;
LARGE_INTEGER	*glpFrequency;
LARGE_INTEGER	 gStartTime;
LARGE_INTEGER	 gEndTime;

eb_t	myBundle		= NULL;
ep_t	myEndpoint		= NULL;
en_t	myEndpointName	= NULL;

unsigned int gTotalExecutions = 16;
unsigned int gTestCount		  = 0;
unsigned int gDataAmount	  = 0;

// Main for test.
void main(int argc, char **argv)
{
	bool	server = false;

	// Intro message
	printf("AMTEST: Test-Active-Messages.");
	#ifdef _DEBUG
		printf(" [DEBUG]\n");
	#else
		printf(" [RELEASE]\n");
	#endif

	// Help message.
	if(argc < 4)
	{
		printf("Usage: amtest <test_type> <name_server_node> <server | client> <param> <param> <...>\n");
		printf("Test Types:\n");
		printf("round_trip      : p1 = num iter\n");
		printf("round_trip_xfer : p1 = num iter, p2 = buffersize\n");
		printf("vi_round_trip   : p1 = num iter, p2 = remote addr, p3 = bufsize\n");
		printf("round_trip_i    : p1 = num iter, p2 = bufsize\n");
		printf("\n");
		exit(1);
	}

	// Set up high res timer.
	glpFrequency = &gFrequency;
	if(QueryPerformanceFrequency(glpFrequency))
	{
		printf("Gots a high res timer: Frequency= %d pulses/second\n", gFrequency.QuadPart);
	}

	// See if this is the server.
	if(strcmp(argv[3], "server") == 0)
	{
		server = true;
	} 

	// Run the appropriate test.
	if(strcmp(argv[1], "round_trip") == 0)
	{
		printf("Running test: round_trip...\n");
		RoundTripTest(argv, server);
	} 
	else if(strcmp(argv[1], "round_trip_xfer") == 0)
	{
		printf("Running test: round_trip_xfer...\n");
		RoundTripTestXfer(argv, server);	
	}
	else if(strcmp(argv[1], "round_trip_i") == 0)
	{
		printf("Running test: round_trip_i...\n");
		RoundTripTestI(argv, server);
	}
	else if(strcmp(argv[1], "vi_round_trip") == 0)
	{
		printf("Running test: vi_round_trip...\n");
		VIRoundTripTest(argc, argv, server);
	}
	else
	{
		printf("Error: Unknown test type (%s)\n", argv[1]);
		exit(1);
	}
} /* end main */

// Round trip test not sending any data.
void RoundTripTest(char **argv, bool server)
{
	int retVal;
	int	rc;

	gTotalExecutions = atoi(argv[4]);
	printf("Total Iterations: %d\n", gTotalExecutions);

	// Initialize AM
	if(AM_Init() != AM_OK)
	{
		printf("Error: Unable to initialize am.\n");
	}

	// Initialize the name service.
	if(AMNS_Init(argv[2]) != AM_OK)
	{
		printf("Error: Unable to initialize am name service.\n");
	}

	// Allocate a bundle.
	rc = AM_AllocateBundle(AM_SEQ, &myBundle);
	if(rc != AM_OK)
	{
		printf("Error: Unable to alloc bundle. (rc=%d)\n", rc);
		AM_Error("yup, couldn't alloc bundle.", __LINE__, __FILE__, rc);
	}

	// Allocate an endpoint.
	if(AM_AllocateEndpoint(myBundle, &myEndpoint, &myEndpointName) != AM_OK)
	{
		printf("Error: Unable to alloc endpoint.\n");
	}

	// Set the endpoint tag.
	AM_SetTag(myEndpoint, AM_ALL);

	// Set the expected resources.
	AM_SetExpectedResources(myEndpoint, 8, 10);

	// Name service stuff.
	if(server)
	{
		if(AMNS_Register("round_trip_test", myEndpointName) != AM_OK)
		{
			printf("Error: Unable to register myEndpoint.\n");
		}
		else
		{
			printf("Register success!\n");
		}
	}

	// Set the appropriate handlers.
	if(AM_SetHandler(myEndpoint, RETURN_TO_SENDER, (Function)ReturnToSender) != AM_OK)
	{
		printf("Error: Unable to set rts handler.\n");
	}
	if(AM_SetHandler(myEndpoint, ROUND_TRIP_REQUEST, (Function)RoundTripRequest) != AM_OK)
	{
		printf("Error: Unable to set round-trip-request handler.\n");
	}
	if(AM_SetHandler(myEndpoint, ROUND_TRIP_REPLY, (Function)RoundTripReply) != AM_OK)
	{
		printf("Error: Unable to set handler2.\n");
	}

	// Set the error output stream
	AM_SetErrorStream(cerr);

	if(!server)
	{
		_en_t	vName;
		if(AMNS_Lookup("round_trip_test", &vName) != AM_OK)
		{
			printf("Unable to find remote endpoint name '%s'. (NS RUNNING?)\n", "round_trip_test");
		}
		else
		{
			printf("Found remote endpoint name.\n");
			printf("Endpoint:%s:%s:%s:%d\n", vName.node, vName.unique, vName.bundle, vName.endpoint);
		}	

		// Map in the server.
		if(AM_Map(myEndpoint, 2, &vName, 0) != AM_OK)
		{
			printf("Error: Unable to map.\n");
		}

		// Lock the test mutex.
		gTestMutex.Lock();
		fflush(stdout);

		// Start timing!
		gStartTick = GetTickCount();
		QueryPerformanceCounter(&gStartTime);

		// Start the test.
		retVal = AM_Request4(myEndpoint, 2, ROUND_TRIP_REQUEST, 3, 2, 1, 0);
		if(retVal != AM_OK)
		{
			AM_Error("Unable to send active message to start test.", __LINE__, __FILE__, retVal);
			gTestMutex.Unlock();
		}
		gTestMutex.Lock();	// Go to sleep
		printf("<<DONE: %d round-trips in [%d ms | %d pulses]!!>>\n", gTestCount+1, gEndTick-gStartTick, gEndTime.QuadPart-gStartTime.QuadPart);
	}
	else
	{
		// Server
		printf("Press <enter> to exit.\n");
		char	c;
		scanf("%c", &c);
	}

	// Free the endpoint.
	if(AM_FreeEndpoint(myEndpoint) != AM_OK)
	{
		printf("Error: Unable to free my endpoint.\n");
	}

	// Free the bundle.
	if(AM_FreeBundle(myBundle) != AM_OK)
	{
		printf("Error: Unable to free bundle.\n");
	}

	// Close AM
	if(AM_Terminate() != AM_OK)
	{
		printf("Error: Unable to close am.\n");
	}
}


// Round trip test sending x amount of data
void RoundTripTestI(char **argv, bool server)
{
	int   retVal;
	int	  rc;
	char *buffer;

	gDataAmount		 = atoi(argv[5]);
	gTotalExecutions = atoi(argv[4]);
	printf("Total Iterations: %d\n", gTotalExecutions);
	printf("Data Size: %d\n", gDataAmount);

	// Make sure buffer is small enough
	if(gDataAmount > (unsigned int)AM_MaxMedium())
	{
		printf("Error: Buffer is larger than AM transfer size.\n");
		exit(1);
	}

	// Create the buffer
	buffer = new char[gDataAmount];
	if(!buffer)
	{
		printf("Unable to alloc buffer for test.\n");
		exit(1);
	}

	// Initialize AM
	if(AM_Init() != AM_OK)
	{
		printf("Error: Unable to initialize am.\n");
	}

	// Initialize the name service.
	if(AMNS_Init(argv[2]) != AM_OK)
	{
		printf("Error: Unable to initialize am name service.\n");
	}

	// Allocate a bundle.
	rc = AM_AllocateBundle(AM_SEQ, &myBundle);
	if(rc != AM_OK)
	{
		printf("Error: Unable to alloc bundle. (rc=%d)\n", rc);
		AM_Error("yup, couldn't alloc bundle.", __LINE__, __FILE__, rc);
	}

	// Allocate an endpoint.
	if(AM_AllocateEndpoint(myBundle, &myEndpoint, &myEndpointName) != AM_OK)
	{
		printf("Error: Unable to alloc endpoint.\n");
	}

	// Set the endpoint tag.
	AM_SetTag(myEndpoint, AM_ALL);

	// Name service stuff.
	if(server)
	{
		if(AMNS_Register("round_trip_test", myEndpointName) != AM_OK)
		{
			printf("Error: Unable to register myEndpoint.\n");
		}
		else
		{
			printf("Register success!\n");
		}
	}

	// Set the appropriate handlers.
	if(AM_SetHandler(myEndpoint, RETURN_TO_SENDER, (Function)ReturnToSender) != AM_OK)
	{
		printf("Error: Unable to set rts handler.\n");
	}
	if(AM_SetHandler(myEndpoint, ROUND_TRIP_REQUESTI, (Function)RoundTripRequestI) != AM_OK)
	{
		printf("Error: Unable to set round-trip-request handler.\n");
	}
	if(AM_SetHandler(myEndpoint, ROUND_TRIP_REPLYI, (Function)RoundTripReplyI) != AM_OK)
	{
		printf("Error: Unable to set handler2.\n");
	}

	// Set the error output stream
	AM_SetErrorStream(cerr);

	if(!server)
	{
		_en_t	vName;
		if(AMNS_Lookup("round_trip_test", &vName) != AM_OK)
		{
			printf("Unable to find remote endpoint name '%s'. (NS RUNNING?)\n", "round_trip_test");
		}
		else
		{
			printf("Found remote endpoint name.\n");
			printf("Endpoint:%s:%s:%s:%d\n", vName.node, vName.unique, vName.bundle, vName.endpoint);
		}	

		// Map in the server.
		if(AM_Map(myEndpoint, 2, &vName, 0) != AM_OK)
		{
			printf("Error: Unable to map.\n");
		}

		// Lock the test mutex.
		gTestMutex.Lock();
		fflush(stdout);

		// Start timing!
		gStartTick = GetTickCount();
		QueryPerformanceCounter(&gStartTime);

		// Start the test.
		retVal = AM_RequestI4(myEndpoint, 2, ROUND_TRIP_REQUESTI, buffer, gDataAmount, 3, 2, 1, 0);
		if(retVal != AM_OK)
		{
			AM_Error("Unable to send active message to start test.", __LINE__, __FILE__, retVal);
			gTestMutex.Unlock();
		}
		gTestMutex.Lock();	// Go to sleep
		printf("<<DONE: %d round-trips in [%d ms | %d pulses]!!>>\n", gTestCount+1, gEndTick-gStartTick, gEndTime.QuadPart-gStartTime.QuadPart);
	}
	else
	{
		// Server
		printf("Press <enter> to exit.\n");
		char	c;
		scanf("%c", &c);
	}

	// Free the endpoint.
	if(AM_FreeEndpoint(myEndpoint) != AM_OK)
	{
		printf("Error: Unable to free my endpoint.\n");
	}

	// Free the bundle.
	if(AM_FreeBundle(myBundle) != AM_OK)
	{
		printf("Error: Unable to free bundle.\n");
	}

	// Close AM
	if(AM_Terminate() != AM_OK)
	{
		printf("Error: Unable to close am.\n");
	}
}

// Round trip test sending x amount of data using xfers
void RoundTripTestXfer(char **argv, bool server)
{
	int   retVal;
	int	  rc;
	char *buffer;

	gDataAmount		 = atoi(argv[5]);
	gTotalExecutions = atoi(argv[4]);
	printf("Total Iterations: %d\n", gTotalExecutions);
	printf("Data Size: %d\n", gDataAmount);

	// Make sure buffer is small enough
	if(gDataAmount > (unsigned int)AM_MaxLong())
	{
		printf("Error: Buffer is larger than AM long xfer size.\n");
		exit(1);
	}

	// Create the buffer
	buffer = new char[gDataAmount];
	if(!buffer)
	{
		printf("Unable to alloc buffer for test.\n");
		exit(1);
	}

	// Initialize AM
	if(AM_Init() != AM_OK)
	{
		printf("Error: Unable to initialize am.\n");
	}

	// Initialize the name service.
	if(AMNS_Init(argv[2]) != AM_OK)
	{
		printf("Error: Unable to initialize am name service.\n");
	}

	// Allocate a bundle.
	rc = AM_AllocateBundle(AM_SEQ, &myBundle);
	if(rc != AM_OK)
	{
		printf("Error: Unable to alloc bundle. (rc=%d)\n", rc);
		AM_Error("yup, couldn't alloc bundle.", __LINE__, __FILE__, rc);
	}

	// Allocate an endpoint.
	if(AM_AllocateEndpoint(myBundle, &myEndpoint, &myEndpointName) != AM_OK)
	{
		printf("Error: Unable to alloc endpoint.\n");
	}

	// Set the endpoint tag.
	AM_SetTag(myEndpoint, AM_ALL);

	// Set the segment
	AM_SetSeg(myEndpoint, buffer, gDataAmount);

	// Name service stuff.
	if(server)
	{
		if(AMNS_Register("round_trip_xfer_test", myEndpointName) != AM_OK)
		{
			printf("Error: Unable to register myEndpoint.\n");
		}
		else
		{
			printf("Register success!\n");
		}
	}

	// Set the appropriate handlers.
	if(AM_SetHandler(myEndpoint, RETURN_TO_SENDER, (Function)ReturnToSender) != AM_OK)
	{
		printf("Error: Unable to set rts handler.\n");
	}
	if(AM_SetHandler(myEndpoint, ROUND_TRIP_REQUESTXFER,  (Function)RoundTripRequestXfer) != AM_OK)
	{
		printf("Error: Unable to set round-trip-request-xfer handler.\n");
	}
	if(AM_SetHandler(myEndpoint, ROUND_TRIP_REPLYXFER, (Function)RoundTripReplyXfer) != AM_OK)
	{
		printf("Error: Unable to set round trip reply xfer.\n");
	}

	// Set the error output stream
	AM_SetErrorStream(cerr);

	if(!server)
	{
		_en_t	vName;
		if(AMNS_Lookup("round_trip_xfer_test", &vName) != AM_OK)
		{
			printf("Unable to find remote endpoint name '%s'. (NS RUNNING?)\n", "round_trip_test");
		}
		else
		{
			printf("Found remote endpoint name.\n");
			printf("Endpoint:%s:%s:%s:%d\n", vName.node, vName.unique, vName.bundle, vName.endpoint);
		}	

		// Map in the server.
		if(AM_Map(myEndpoint, 2, &vName, 0) != AM_OK)
		{
			printf("Error: Unable to map.\n");
		}

		// Lock the test mutex.
		gTestMutex.Lock();
		fflush(stdout);

		// Start timing!
		gStartTick = GetTickCount();
		QueryPerformanceCounter(&gStartTime);

		// Start the test.
		retVal = AM_RequestXfer4(myEndpoint, 2, 0, ROUND_TRIP_REQUESTXFER, buffer, gDataAmount, 3, 2, 1, 0);
		if(retVal != AM_OK)
		{
			AM_Error("Unable to send active message to start test.", __LINE__, __FILE__, retVal);
			gTestMutex.Unlock();
		}
		gTestMutex.Lock();	// Go to sleep
		printf("<<DONE: %d round-trips in [%d ms | %d pulses]!!>>\n", gTestCount+1, gEndTick-gStartTick, gEndTime.QuadPart-gStartTime.QuadPart);
	}
	else
	{
		// Server
		printf("Press <enter> to exit.\n");
		char	c;
		scanf("%c", &c);
	}

	// Free the endpoint.
	if(AM_FreeEndpoint(myEndpoint) != AM_OK)
	{
		printf("Error: Unable to free my endpoint.\n");
	}

	// Free the bundle.
	if(AM_FreeBundle(myBundle) != AM_OK)
	{
		printf("Error: Unable to free bundle.\n");
	}

	// Close AM
	if(AM_Terminate() != AM_OK)
	{
		printf("Error: Unable to close am.\n");
	}
	delete[] buffer;
}

void VIRoundTripTest(int argc, char **argv, bool server)
{
	cCQ			*cq			= NULL;
	cNic		*nic		= NULL;
	cVI			*vi			= NULL;
	cPtag		*ptag		= NULL;
	cDesc		*descIn		= NULL;
	cDesc		*descOut	= NULL;
	cVIAddr		*localAddr	= NULL;
	cVIAddr		*remoteAddr	= NULL;
	cDSeg		*dsegIn		= NULL;
	cDSeg		*dsegOut	= NULL;

	DWORD	startTick;
	DWORD	endTick;


	gTotalExecutions = atoi(argv[4]);
	printf("Number of iterations: %d\n", gTotalExecutions);

	if(argc > 6)
	{
		gDataAmount = atoi(argv[6]);
	}
	else
	{
		gDataAmount = 0;
	}
	printf("Data Size: %d\n", gDataAmount);

	nic = new cNic(GIGANET_DEVICE_NAME);
	if(!(nic->IsOpen()))
	{
		printf("Test Error: Unable to open nic.\n");
		exit(1);
	}
	ptag = nic->CreatePtag();

	cq = nic->CreateCQ(128);

	VIP_VI_ATTRIBUTES		  viAttrib;
	const VIP_NIC_ATTRIBUTES *nicAttrib;
	nicAttrib = nic->GetAttrib();
	viAttrib.ReliabilityLevel = VIP_SERVICE_UNRELIABLE;
	viAttrib.MaxTransferSize  = nicAttrib->MaxTransferSize;
	viAttrib.QoS			  = 0;
	viAttrib.Ptag			  = ptag->GetPtag();
	viAttrib.EnableRdmaWrite  = VIP_FALSE;
	viAttrib.EnableRdmaRead   = VIP_FALSE;

	vi = nic->CreateVI(&viAttrib, cq, cq);


	descIn = nic->CreateDesc(2, ptag);
	descOut = nic->CreateDesc(2, ptag);

	localAddr = nic->CreateVIAddr();
	localAddr->GetLocalAddress();
	remoteAddr = nic->CreateVIAddr();
	remoteAddr->GetHostByName(argv[5]);


	// Create and fill-out a data segment
	if(gDataAmount > 0)
	{
		dsegIn = nic->CreateDSeg(gDataAmount, ptag, VIP_FALSE, VIP_FALSE);
		descIn->SetDSeg(0, dsegIn);
		descIn->SetLength(dsegIn->GetSize());
		descIn->SetSegCount(1);
	} 
	else
	{
		descIn->SetSegCount(0);
		descIn->SetLength(0);
	}
	descIn->SetControl(VIP_CONTROL_OP_SENDRECV);

	if(gDataAmount > 0)
	{
		dsegOut = nic->CreateDSeg(gDataAmount, ptag, VIP_FALSE, VIP_FALSE);
		descOut->SetDSeg(0, dsegOut);
		descOut->SetLength(dsegOut->GetSize());
		descOut->SetSegCount(1);
	}
	else
	{
		descOut->SetSegCount(0);
		descOut->SetLength(0);
	}
	descOut->SetControl(VIP_CONTROL_OP_SENDRECV);

	if(server)
	{
		/** I AM SERVER **/
		VIP_RETURN rc;
		VIP_DESCRIPTOR *descPtr;
		VIP_VI_HANDLE   viHandle;
		VIP_BOOLEAN		recvQueue;

		// Post Desc. on recv queue.
		descIn->SetControl(VIP_CONTROL_OP_SENDRECV);
		vi->PostRecv(descIn);

		// Connect
		printf("Waiting for connection.\n");
		rc = vi->ConnectWait(localAddr, remoteAddr, 30000);
		if(rc != VIP_SUCCESS)
		{
			printf("Test Error: ConnectWait failed!");
		}
		else
		{
			rc = vi->ConnectAccept();
			if(rc != VIP_SUCCESS)
			{
				printf("Test Error: ConnectAccept failed!");
			}
			else
			{
				printf("Yess!! Successfully connected.\n");

				for(unsigned int i = 0; i < gTotalExecutions; i++)
				{
					do 
					{
						//rc = vi->RecvDone(descIn);
						//rc = cVI::RecvDone(vi->GetHandle(), &descPtr);
						rc = cq->Done(&viHandle, &recvQueue);
					} while(rc == VIP_NOT_DONE);
					rc = cVI::RecvDone(vi->GetHandle(), &descPtr);
					if(rc != VIP_SUCCESS)
					{
						printf("Test Error: Receive gone bad!\n");
					}
					rc = vi->PostRecv(descIn);
					rc = vi->PostSend(descOut);
					do
					{
						//rc = vi->SendDone(descOut);
						//rc = cVI::SendDone(vi->GetHandle(), &descPtr);
						rc = cq->Done(&viHandle, &recvQueue);
					} while(rc == VIP_NOT_DONE);
					rc = cVI::SendDone(vi->GetHandle(), &descPtr);
				}
				printf("Disconnecting.\n");
				vi->Disconnect();
			}
		}
	}
	else
	{
		/** I AM CLIENT **/
		VIP_RETURN rc;
		VIP_VI_HANDLE   viHandle;
		VIP_BOOLEAN		recvQueue;

		printf("Requesting connection to %s\n", argv[1]);
		rc = vi->PostRecv(descIn);
		rc = vi->ConnectRequest(localAddr, remoteAddr, 30000);
		if (rc != VIP_SUCCESS)
		{
			printf("Test Error: Unable to connect to remote server!\n");
		}
		else
		{
			printf("Connected to server!\n");
			VIP_DESCRIPTOR *descPtr;

			// START TIMING
			startTick = GetTickCount();
			for(unsigned int j = 0; j < gTotalExecutions; j++)
			{
				// Post Send Desc.
				rc = vi->PostSend(descOut->GetDescPtr(), descOut->GetMemHandle());
				do
				{
					//rc = vi->SendDone(descOut);
					//rc = cVI::SendDone(vi->GetHandle(), &descPtr);
					rc = cq->Done(&viHandle, &recvQueue);
				} while(rc == VIP_NOT_DONE);
				rc = cVI::SendDone(vi->GetHandle(), &descPtr);

				if(rc != VIP_SUCCESS)
				{
					printf("Test Error: Send wait gone bad!\n");
				}
				do
				{
					//rc = vi->RecvDone(descIn);
					//rc = cVI::RecvDone(vi->GetHandle(), &descPtr);
					rc = cq->Done(&viHandle, &recvQueue);
				} while(rc == VIP_NOT_DONE);
				rc = cVI::RecvDone(vi->GetHandle(), &descPtr);

				if(rc != VIP_SUCCESS)
				{
					printf("Test Error: Receive gone bad!\n");
				}
				vi->PostRecv(descIn->GetDescPtr(), descIn->GetMemHandle());
			}
			endTick = GetTickCount();
			printf("<<DONE: %d round-trips in [%d ms]!!>>\n", j, endTick-startTick);
		}
	}

	printf("Disconecting.\n");
	vi->Disconnect();
	vi->Flush();

	printf("Deleting objects (in reverse order)...\n");

		printf("Deleting remoteAddr.\n");
		fflush(stdout);
		if(remoteAddr) { delete remoteAddr; }
		printf("Deleting localAddr.\n");
		fflush(stdout);
		if(localAddr) { delete localAddr; }
		printf("Deleting descs.\n");
		fflush(stdout);
		if(descIn)  { delete descIn;  }
		if(descOut) { delete descOut; }
		if(dsegOut) { delete dsegOut; }
		if(dsegIn)  { delete dsegIn;  }
		printf("Deleting completion queue.\n");
		if(cq) { delete cq; }
		printf("Deleting vi.\n");
		fflush(stdout);
		if(vi) { delete vi; }
		printf("Deleting ptag.\n");
		fflush(stdout);
		if(ptag) { delete ptag; }
		printf("Deleting nic.\n");
		fflush(stdout);
		if(nic) { delete nic; }

	printf("Done!\n");
}
