#ifndef __C_BUNDLE_H
#define __C_BUNDLE_H
/*
 * cBundle.h
 * 
 * Class def for AM bundle class.
 *
 */
#include "am.h"
#include "stdam.h"
#include <windows.h>

#include "cStack.h"
#include "cFifo.h"
#include "cPacket.h"

#define _LISTEN_TIMEOUT		3000	// Time spent listening for connection.
#define _EXTINCTION_TIMEOUT	5000	// Time until a connection is killed for good.

class cAMLayer;
class cEndpoint;

//cBundle class
class cBundle
{

 protected:

 public:
		
	//Constructor(s)
	cBundle(cAMLayer *layer, int number);	

	//Initialize
	bool	Init();

	//Destructor
	~cBundle();

	//Thread Control
	bool	Start();
	bool	Stop();
	bool	IsRunning();

	//Methods
	int			AllocateEndpoint(cEndpoint **ep, en_t *endpointName);
	int			FreeEndpoint(cEndpoint *ep);
	char*		GetBundleName(int *nameLength);

	cFifo*		GetConnectionQ() { return mUsedConnectionQ; }

	int			GetNumber();
	cEndpoint*	GetEndpoint(int number) { return ((number < mEndpointArraySize) ? mEndpointArray[number] : NULL); }

	cConnection* GetNewConnection();
	bool		AddToClosedConnectionQueue(cConnection *connection) { return (mClosedConnectionQ->Push(connection) != NULL); }
	void		ReleaseConnection(cConnection *connection);
	cFifo*		GetUsedConnectionQ()	{ return mUsedConnectionQ; }

	cPtag*		GetPtag()	{ return mPtag;					}
	cCQ*		GetCQ()		{ return mCQ;					}
	cNic*		GetNic()	{ return mNic;					}

	//Internal
	DWORD		_BundleMain();
	DWORD		_ConnectMain();

	cVIAddr		 *mLocalAddress;		// Pointer to the local bundle address.

	_en_t		  mBundleAMAddr;		// The AM address of this bundle.

 private:

	// Private functions.
	bool	_StopMain();
	bool	_StopConnect();
	void	_HandleSend(cConnection *connection, cPacket *packet);

	// -Thread Vars-
	HANDLE		  mMainThreadHandle;	// thread handle of the main thread.
	DWORD		  mMainThreadId;		// id of the main thread.
	HANDLE		  mConnectThreadHandle;	// thread handle of the connect thread.
	DWORD		  mConnectThreadId;		// id of the connect thread.
	volatile bool mRunMain;				// Flag: true->running, false->dead
	volatile bool mRunConnection;		//  ""

	//Member Vars
	int			  mNumber;              // This bundle's location in the AM Layer.
	cEndpoint	**mEndpointArray;		// Array to hold active endpoints in this bundle.
	int			  mEndpointArraySize;	// Current max number of endpoints.
	cAMLayer	 *mAMLayer;				// Pointer to the AM layer that created this bundle.

	//Resources
	cFifo		 *mUsedConnectionQ;		// A queue full of used connection resources.
	cFifo		 *mClosedConnectionQ;	// A queue full of closed connections awaiting extinction.

	//VI things.
	cNic		 *mNic;					// The nic.
	cPtag		 *mPtag;				// The Ptag
	cCQ			 *mCQ;					// The completion queue.

	int			  mNumPacketsInUse;		// Number of packets in use for debug
	int			  mAcks;

	// Locks
 	cCriticalSection	mCSEndpointArray;
};

#endif
