#ifndef __C_CONNECTION_H
#define __C_CONNECTION_H

#include "stdam.h"

#include "Header.h"
#include "cCriticalSection.h"
#include "Node.h"
#include "cStack.h"

class cBundle;
class cEndpoint;
class cPacket;

#define DEFAULT_WINDOW_SIZE	8

#define _CONNECT_STATE_WAITING 0
#define _CONNECT_STATE_SUCCESS 1
#define _CONNECT_STATE_FAIL    2

/*
 * cConnection.h
 * 
 * Connection class.
 *
 */
class cConnection
{

 protected:

 public:
		
	// Constructor(s)
	cConnection(cNic *nic, cCQ *cq, cBundle *bundle, cEndpoint *endpoint, int outstandingReq=DEFAULT_WINDOW_SIZE);

	// Destructor.
	~cConnection();

	// Public functions.
	int		SendPacket(cPacket *packet);
	int		RecvPacket(cPacket *packet);

	cPacket* GetFreePacket() { return (cPacket *)mFreePacketQ->Pop(); }
	void	 ReleasePacket(cPacket *packet);

	DWORD	 GetTimeStamp() { return mTimeStamp; }

	bool	OpenConnection(en_t remoteAddress);
	void	DisableConnection()	{ mEnabled = false; }
	bool	CloseConnection();
	bool	IsExtinct();
	bool	IsActive()	{ return mActive; }
	bool	PostRecv();

	VIP_RETURN	Listen(cVIAddr *localAddress, long timeout);	// Listen for a new connection.

	en_t		GetAMAddress()	{ return &mAMAddress; }
	void		SetAMAddress(en_t addr);
	void		SetEndpoint(cEndpoint *endpoint) { mEndpoint = endpoint; }
	cBundle*	GetBundle()		{ return mBundle; }
	void		RecvQEmptyWarning()	{ ; }

	// Internal data
	cVI			*mVI;
	cVIAddr		*mVIAddress;
	Node		*mNode;

 private:

	// Internal functions.
	cVI*		_MakeVI(cNic *nic, cCQ *cq);
	int			_AdvanceReceiveWindow();
	int			_HandleAck(Header *header);
	int			_SendAck();
	int			_FinishConnection(cPacket *packet, int *currentPtr);
	bool		_ResizeWindows(unsigned int newSize, int *currentPtr);


	// Remote address
	_en_t		 mAMAddress;

	cBundle		*mBundle;
	cEndpoint	*mEndpoint;

	bool	mActive;
	bool	mEnabled;
	volatile char	mConnectStatus;

	// Sequence numbers.
	unsigned int mSeqOut;
	unsigned int mCumSeqIn;
	unsigned int mCumSeqOut;

	// Send window
	cPacket		  **mSendWindow;
	unsigned int	mSendWindowInsert;
	unsigned int	mSendWindowRemove;
	unsigned int	mSendWindowSize;
	unsigned int	mSendWindowAvailable;

	// Recv window
	cPacket		  **mRecvWindow;
	unsigned int	mRecvWindowPtr;
	unsigned int	mRecvWindowSize;

	// Free packets
	cStack		   *mFreePacketQ;
	unsigned int	mNumPackets;

	// Mutexs
	cCriticalSection	mSendCS;
	cCriticalSection	mRecvCS;

	// Close connection time stamp.
	DWORD				mTimeStamp;

	// Number of packets posted on the recv queue.
	unsigned int	mRecvPacketsPosted;

};

#endif