#ifndef __C_ENDPOINT_H
#define __C_ENDPOINT_H
/*
 * cEndpoint.h
 * 
 * Class def for AM endpoint class.
 *
 */
#include "am.h"
#include "stdam.h"
#include "cCriticalSection.h"
class cBundle;
class cConnection;

typedef struct 
{
	tag_t			 tag;		 // The tag.
	int				 number;	 // The endpoint number.
	cConnection		*connection; // The connection.
} MapEntry;

typedef struct
{
	void (*function)();	
} HandlerEntry;

//cEndpoint class
class cEndpoint
{

 protected:

 public:
		
	//Constructor(s)
	cEndpoint(cBundle *mBundle, int number);	

	//Destructor
	~cEndpoint();

	//Public Functions
	int SetExpectedResources(int n_endpoints, int n_outstanding_requests);
	int	GetExpectedMaxNumRequests()	{ return mNumOutstandingRequests; }
	int	Map(int index, en_t epn, tag_t tag);
	int MapAny(int *index, en_t endpoint, tag_t tag);
	int Unmap(int index);
	int SetHandler(handler_t handler, void  (*function)());
	int SetHandlerAny(handler_t *handler, void (*function)());

	void  SetSegment(void *segment, int nbytes) { mSegment = segment;  mSegmentSize = nbytes; } 
	int   GetSegmentSize()		  { return mSegmentSize; }
	void* GetSegment()			  { return mSegment;     }

	int Request4(int reply_endpoint, handler_t handler, int a0, int a1, int a2, int a3);
	int RequestI4(int reply_endpoint, handler_t handler, void *source, int nbytes, int a0, int a1, int a2, int a3);
	int RequestXfer4(int reply_endpoint, int dest_offset, handler_t handler, void *source, int nbytes, int a0, int a1, int a2, int a3);
	int	Reply4(void *token, handler_t handler, int a0, int a1, int a2, int a3);
	int ReplyI4(void *token, handler_t handler, void *source, int nbytes, int a0, int a1, int a2, int a3);
	int ReplyXfer4(void *token, int dest_offset, handler_t handler, void *source, int nbytes, int a0, int a1, int a2, int a3);

	int			ProcessXferIntermediate(cPacket *packet);
	int			ProcessRequest(cPacket *packet);
	int			ProcessReply(cPacket *packet);
	int			ProcessRTS(cPacket *packet);

	int		 GetNumber();
	cBundle	*GetBundle();
	void*    GetFunction(int index) { return mHandlerTable[index].function; }
	en_t	 GetAddress();
	void	 SetAddress(_en_t *addr);

	//Misc
	int		SetTag(tag_t tag) { mTag = tag; return AM_OK; }
	tag_t	GetTag()		  { return mTag;			  }

	int		SetNumHandlers(unsigned int newSize);
	int		GetNumHandlers()	{ return mHandlerTableSize; }
	
	int		GetTranslationName(int i, en_t *gan);
	tag_t	GetTranslationTag(int i)	{ return mMap[i].tag; }
	bool	GetTranslationInUse(int i)	{ return (mMap[i].connection != NULL); }
	int		GetNumTranslations()		{ return mMapSize; }
//int AM_SetNumTranslations(ep_t ea, int ntrans);

 private:

	// Member Vars
	cBundle			*mBundle;
	int				 mNumber;
	MapEntry		*mMap;
	unsigned int	 mMapSize;
	HandlerEntry	*mHandlerTable;
	unsigned int	 mHandlerTableSize;
	_en_t			 mAddress;
	tag_t			 mTag;
	int				 mNumOutstandingRequests;
	void			*mSegment;
	int				 mSegmentSize;

	// Locks
	cCriticalSection	mCSMap;
	cCriticalSection	mCSHandler;
};

#endif
