#ifndef __CFIFO_H__
#define __CFIFO_H__

#include "Node.h"
#include "cCriticalSection.h"

// Fifo exception class.
class cFifoException { 

 public: 
	cFifoException(char *buf)
	{
		strncpy(mString, buf, 127);
		mString[127] = '\0';
	}
	~cFifoException() { ; }

	char *ToString()
	{
		return mString;
	}
 private:
	char	mString[128];
};

//cFifo class
class cFifo
{

 protected:

 public:
		
	//Constructor(s)
	cFifo(int size);	

	//Destructor
	~cFifo();

	//Public Functions
	Node*	Push(void *data);
	bool	Pop(void **data);
	Node*	Peek(void **data);		// Peeks at data of next element to remove.
	bool	Delete(Node *node);
	Node*	GetSentinel();			// Gets the sentinel --> So that outside user can traverse the list.
	void	Lock()  { mCS.Enter(); }// Lock this data structure.
	void	Unlock(){ mCS.Leave(); }// Unlocks this data structure.
	int		GetNumElements()	{ return mNumElements; }

	// Mutex
	cCriticalSection	mCS;

 private:
	 void	_PushFreeList(Node *node);
	 Node*	_PopFreeList();
	 void	_ResetNode(Node *node);

	//Member Vars
	Node	 *mFreeList;
	Node	 *mBack;
	Node	 *mSentinel;
	int		  mSize;
	int		  mNumElements;
};

#endif
