/*
 * cNameServ.cpp
 * 
 * Class def for name server *
 */
#include "cNS.h"

#define TABLESIZE	64

// constructor
cNS::cNS()
{
	try 
	{
		this->NS_Table = new cHashTable(TABLESIZE);
	}
	catch (cHashException &e)
	{
		printf(e.ToString());
		printf("\n");
	}
	if(!NS_Table)
	{
		printf("Error: Could not create hash table.\n");
	}
}

bool cNS::NS_Lookup(char* desc, _en_t *endpt)
{
	Node* check = this->NS_Table->Find(desc, (void**)&endpt);
	if (!check)
	{
		return false;
	}
	return true;
}

bool cNS::NS_Register(char* desc, en_t endpt)
{
	void	*test;
	Node* check = this->NS_Table->Find(desc, (void**)&test);
	if (check)
	{
		bool bCheck =	this->NS_Unregister(desc);
		if (!bCheck) 
		{
			return false;
		}
	}
	check = this->NS_Table->Insert(desc, endpt);
	if (!check)
	{
		return false;
	}
	return true;
}

bool cNS::NS_Unregister(char* desc)
{
	void	*key;
	void	*data;
	Node	*node;

	this->NS_Table->Lock();

	node = this->NS_Table->Find(desc, &data);
	if(node)
	{
		key = node->key;
		this->NS_Table->Delete(node);
		if(key)	 { delete key;  }
		if(data) { delete data; }
		this->NS_Table->Unlock();
		return true;
	}
	this->NS_Table->Unlock();
	return false;
}

cNS::~cNS()
{
	void* key;
	void* data;

	// need to send a msg to readers/writers that they are no 
	// longer registered
	// for now, assume ns only destroyed when all members are removed.
	while(this->NS_Table->DeleteFirst(&key, &data))
	{
		if(key) {delete key;}
		if(data){delete data;}
	}

	delete this->NS_Table;	
}



