#include "cNS.h"
#include <stdio.h>
#include <stdlib.h>
#include <vi/stdvi.h>
#include <am.h>

#define	GIGANET_DEVICE_NAME		"nic0"
#define MAX_NAME_SIZE			128

#define NS_LOOKUP		1
#define NS_REGISTER		2
#define NS_UNREGISTER	3
#define NS_ACK			4
#define NS_NACK			5

#define NS_WAIT_TIME	2000

bool	mRun = true;

// Function prototypes.
void ResetConnection(cVI *vi, cDesc *descIn, cDSeg *dsegIn1, cDSeg *dsegIn2);
void RunServer(cNS *ns);

// Main entry point.
int main(int argc, char *argv[])
{
	cNS	*ns;

	ns = new cNS();
	if(!ns)
	{
		printf("Unable to create name service object.\n");
		exit(1);
	}
	printf("Starting name service...\n");
	RunServer(ns);

	// Assuming could get to here.
	if(ns)
	{
		delete ns;
	}
	return 0;
}

// Server execution
void RunServer(cNS *ns)
{
	cNic	*nic;
	cVI		*vi;
	cPtag	*ptag;
	cDesc	*descIn;
	cDesc	*descOut;
	cDSeg	*dsegIn1;
	cDSeg	*dsegIn2;
	cDSeg	*dsegOut;
	cVIAddr *localAddr;
	cVIAddr *remoteAddr;

	int dsegSize;

	// Set up the segment sizes.
	if(sizeof(_en_t) > MAX_NAME_SIZE)
	{
		dsegSize = sizeof(_en_t);
	}
	else
	{
		dsegSize = MAX_NAME_SIZE;
	}

	// Open the nic.
	nic = new cNic(GIGANET_DEVICE_NAME);
	if(!(nic->IsOpen()))
	{
		printf("Error: Unable to open nic.\n");
		exit(1);
	}
	ptag = nic->CreatePtag();

	VIP_VI_ATTRIBUTES		  viAttrib;
	const VIP_NIC_ATTRIBUTES *nicAttrib;
	nicAttrib = nic->GetAttrib();
	viAttrib.ReliabilityLevel = VIP_SERVICE_RELIABLE_DELIVERY;
	viAttrib.MaxTransferSize  = nicAttrib->MaxTransferSize;
	viAttrib.QoS			  = 0;
	viAttrib.Ptag			  = ptag->GetPtag();
	viAttrib.EnableRdmaWrite  = VIP_FALSE;
	viAttrib.EnableRdmaRead   = VIP_FALSE;

	vi = nic->CreateVI(&viAttrib);

	// Create descriptors.
	descIn = nic->CreateDesc(2, ptag);
	descOut = nic->CreateDesc(2, ptag);

	// Create addresses.
	localAddr = nic->CreateVIAddr();
	localAddr->GetLocalAddress();
	localAddr->SetDiscriminator("ns", 2);
	remoteAddr = nic->CreateVIAddr();

	// Create and fill-out a data segment
	dsegIn1 = nic->CreateDSeg(dsegSize, ptag, VIP_FALSE, VIP_FALSE);
	dsegIn2 = nic->CreateDSeg(dsegSize, ptag, VIP_FALSE, VIP_FALSE);

	descIn->SetDSeg(0, dsegIn1);
	descIn->SetDSeg(1, dsegIn2);
	descIn->SetLength(dsegIn1->GetSize() + dsegIn2->GetSize());
	descIn->SetSegCount(2);
	descIn->SetControl(VIP_CONTROL_OP_SENDRECV);

	dsegOut = nic->CreateDSeg(dsegSize, ptag, VIP_FALSE, VIP_FALSE);
	descOut->SetDSeg(0, dsegOut);
	descOut->SetLength(dsegOut->GetSize());
	descOut->SetSegCount(1);
	descOut->SetControl(VIP_CONTROL_OP_SENDRECV | VIP_CONTROL_IMMEDIATE);

	/** I AM SERVER **/
	VIP_RETURN rc;

	// Connect
	while(mRun)
	{
		ResetConnection(vi, descIn, dsegIn1, dsegIn2);			
		printf("Waiting for connection...\n");
		do
		{
			rc = vi->ConnectWait(localAddr, remoteAddr, NS_WAIT_TIME);
		} while(rc != VIP_SUCCESS);
		printf("Handling connection.\n");

		rc = vi->ConnectAccept();
		if(rc != VIP_SUCCESS)
		{
			printf("Error: ConnectAccept failed!");
			ResetConnection(vi, descIn, dsegIn1, dsegIn2);			
			continue;
		}
		rc = vi->RecvWait(descIn, 30000);
		if(rc != VIP_SUCCESS)
		{
			ResetConnection(vi, descIn, dsegIn1, dsegIn2);			
			printf("Error: Receive gone bad!\n");
			continue;
		}
		else
		{
			en_t	address;
			char    key[MAX_NAME_SIZE];
			char   *newKey;

			// Handle request.
			switch(descIn->GetImmediateData())
			{

			case NS_LOOKUP:
				// Do Lookup
				// seg[0] -> char string for name.
				memcpy(key, dsegIn1->GetData(), MAX_NAME_SIZE);
				printf("NS_LOOKUP.  Key=%s\n", key);
				if(ns->NS_Lookup(key, address))
				{
					// outseg[0] -> actual am address
					printf("Returning Endpoint:%s:%s:%s:%d\n", address->node, address->unique, address->bundle, address->endpoint);
					memcpy(dsegOut->GetData(), address, sizeof(_en_t));
					descOut->SetSegCount(1);
					dsegOut->SetSize(sizeof(_en_t));
					descOut->SetDSeg(0, dsegOut);
					descOut->SetLength(sizeof(_en_t));
					descOut->SetImmediateData(NS_ACK);
					descOut->SetControl(VIP_CONTROL_OP_SENDRECV | VIP_CONTROL_IMMEDIATE);
				}
				else
				{
					// Send NACK
					descOut->SetSegCount(0);
					descOut->SetLength(0);
					descOut->SetImmediateData(NS_NACK);
				}
				break;

			case NS_REGISTER:
				descOut->SetSegCount(0);
				descOut->SetLength(0);

				// seg[0] -> char string for name.
				newKey = new char[MAX_NAME_SIZE];
				if(!newKey)
				{
					// Send NACK
					descOut->SetImmediateData(NS_NACK);
				}
				memcpy(newKey, dsegIn1->GetData(), MAX_NAME_SIZE);
				printf("NS_REGISTER.  newKey=%s\n", newKey);
				address = new _en_t;
				if(!address)
				{
					delete newKey;
					// Send NACK
					descOut->SetImmediateData(NS_NACK);
				}

				// seg[1] -> actual am address.
				memcpy(address, dsegIn2->GetData(), sizeof(_en_t));
				printf("Registering Endpoint:%s:%s:%s:%d\n", address->node, address->unique, address->bundle, address->endpoint);
				if(ns->NS_Register(newKey, address))
				{
					// Send ACK
					printf("Register success!\n");
					descOut->SetImmediateData(NS_ACK);
				}
				else
				{
					delete newKey;
					delete address;

					// Send NACK
					descOut->SetImmediateData(NS_NACK);
				}
				break;

			case NS_UNREGISTER:
				descOut->SetSegCount(0);
				descOut->SetLength(0);

				// seg[0] -> char string for name.
				memcpy(key, dsegIn1->GetData(), MAX_NAME_SIZE);
				printf("NS_UNREGISTER.  Key=%s\n", key);
				if(ns->NS_Unregister(key))
				{
					// Send ACK
					descOut->SetImmediateData(NS_ACK);
				}
				else
				{
					// Send NACK
					descOut->SetImmediateData(NS_NACK);
				}
				break;

			}
			printf("Immediate data: %d\n", descOut->GetImmediateData());
			rc = vi->PostSend(descOut);
			if(rc != VIP_SUCCESS)
			{
				printf("Unable to send.  rc=%d\n", rc);
			}
			vi->SendWait(descOut, 20000);
		}
		printf("Disconnecting.\n");
	}
}

/*
 * Resets the connection.
 */
void ResetConnection(cVI *vi, cDesc *descIn, cDSeg *dsegIn1, cDSeg *dsegIn2)
{
	vi->Disconnect();
	vi->Flush();
	descIn->SetLength(dsegIn1->GetSize() + dsegIn2->GetSize());
	descIn->SetSegCount(2);
	descIn->SetControl(VIP_CONTROL_OP_SENDRECV);
	vi->PostRecv(descIn);
}