/*
 * cDesc.cpp
 * 
 * Class def for VI descriptor class. 
 *
 * Authors:  Vicky & Ted
 * Last Mod: 4 March 1999 by ted
 */
#include "cDesc.h"
#include "cDSeg.h"
#include <malloc.h>


//cDesc class
cDesc::cDesc(int numSegments, VIP_MEM_ATTRIBUTES *attrib, VIP_NIC_HANDLE nic)
{	
	VIP_RETURN rc;
	VIP_ULONG numExtraSegs=0;

	// initialize member variables
	mNumSegments = numSegments;
	mNicHandle   = nic;
	mAllocPtr    = NULL;

	// get the memory.  
	//VIP_DESCRIPTOR includes space for 2 descripter segments.  Extra descriptor segments need extra space.    
	if(mNumSegments > 2)
	{
		numExtraSegs= mNumSegments - 2;
	}

	VIP_ULONG totalSize = sizeof(VIP_DESCRIPTOR) + numExtraSegs*sizeof(VIP_DESCRIPTOR_SEGMENT);

	mAllocPtr = calloc(1, totalSize + VIP_DESCRIPTOR_ALIGNMENT);
	if(!mAllocPtr)
	{
		throw VIException("Couldn't allocate memory in Desc constructor.");
	}

	// Align the descriptor.
	mDescPtr = (VIP_DESCRIPTOR *)
				((int)((char *)mAllocPtr + VIP_DESCRIPTOR_ALIGNMENT - 1)&(0-VIP_DESCRIPTOR_ALIGNMENT));

	// register the memory.
	rc = VipRegisterMem(nic, mDescPtr, totalSize, attrib, &mMemHandle);
	if (rc != VIP_SUCCESS)
	{
		throw VIException("Couldn't register memory in cDesc constructor.");
	}
}

/* Data Segments */
VIP_BOOLEAN cDesc::SetDSeg(int number, cDSeg *seg)
{
	mDescPtr->DS[number].Local.Data.Address = seg->GetData();
	mDescPtr->DS[number].Local.Handle = seg->GetMemHandle();
	mDescPtr->DS[number].Local.Length = seg->GetSize();
	return VIP_TRUE;
}

VIP_BOOLEAN cDesc::SetASeg(VIP_ADDRESS_SEGMENT *addr)
{
	mDescPtr->DS[0].Remote.Data.Address = addr->Data.Address;
	mDescPtr->DS[0].Remote.Handle       = addr->Handle;
	return VIP_TRUE;
}


//Destructor
cDesc::~cDesc()
{
	VIP_RETURN rc;

	rc = VipDeregisterMem(mNicHandle, mDescPtr, mMemHandle);
	if(rc != VIP_SUCCESS)
	{
		cVIError::PrintError("Couldn't deregister memory in Desc destructor.", __LINE__, __FILE__, rc);
	}
	free(mAllocPtr);
}
