#ifndef __C_NIC_H
#define __C_NIC_H
/*
 * cNic.h
 * 
 * Class def for VI NIC (Network Interface Connection) object.
 *
 * Authors:  Vicky & Ted
 * Last Mod: 27 Feb 1999
 */
#include "stdvint.h"

class cVI;
class cDesc;
class cDSeg;
class cPtag;
class cVIAddr;
class cCQ;

//cNic class
class cNic
{

	public:
		
		//Constructor(s)
		cNic(const char *DeviceName);

		//Accessor Functions
		VIP_NIC_HANDLE				GetNicHandle() { return mNic; }
		const VIP_NIC_ATTRIBUTES*	GetAttrib();
		VIP_BOOLEAN					IsOpen();
		VIP_RETURN			ErrorCallback(VIP_PVOID Context,
										  void(*Handler) (
													VIP_PVOID			 Context,
													VIP_ERROR_DESCRIPTOR *ErrorDesc
													)
										  );

		//Object Creation Functions
		cVI*		CreateVI(VIP_VI_ATTRIBUTES *attrib);
		cVI*		CreateVI(VIP_VI_ATTRIBUTES *attrib, cCQ *SendCQ, cCQ *RecvCQ);
		cDesc*		CreateDesc(int numSegments, cPtag *ptag, VIP_BOOLEAN RdmaWrite=VIP_FALSE);
		cDSeg*		CreateDSeg(VIP_ULONG length, cPtag *ptag, VIP_BOOLEAN RdmaWrite, VIP_BOOLEAN RdmaRead);
		cDSeg*		CreateDSeg(VIP_ULONG length, void *buffer, cPtag *ptag, VIP_BOOLEAN RdmaWrite, VIP_BOOLEAN RdmaRead);
		cPtag*		CreatePtag();
		cVIAddr*	CreateVIAddr();
		cCQ*		CreateCQ(VIP_ULONG size);


		//Destructor
		~cNic();

	private:

		//Member Vars
		VIP_NIC_HANDLE			mNic;
		VIP_NIC_ATTRIBUTES		mAttrib;
		VIP_BOOLEAN				mOpen;

	protected:


};

#endif