/*
 * cVI.cpp
 * 
 * Class for VI objects.
 *
 * Authors:  Vicky & Ted
 * Last Mod: 4 March 1999 by ted
 */

#include "cVI.h"
#include "cCQ.h"
#include "cDesc.h"
#include "cVIAddr.h"
#include <assert.h>
#include <stdio.h>

//Constructor(s)
cVI::cVI(VIP_NIC_HANDLE nic, VIP_VI_ATTRIBUTES *attribs)
{
	VIP_RETURN	rc;

	mNic = nic;

	rc = VipCreateVi(nic, attribs, NULL, NULL, &mVI);
	if (rc != VIP_SUCCESS)
	{
		throw VIException("Unable to create vi.");
	}
}

cVI::cVI(VIP_NIC_HANDLE nic, VIP_VI_ATTRIBUTES *attribs, cCQ *SendCQ, cCQ *RecvCQ)
{
	VIP_RETURN		rc;
	VIP_CQ_HANDLE	SendCQH;
	VIP_CQ_HANDLE	RecvCQH;

	SendCQH = NULL;
	RecvCQH = NULL;
	if(SendCQ != NULL)
	{
		SendCQH = SendCQ->GetHandle();
	}
	if(RecvCQ != NULL)
	{
		RecvCQH = RecvCQ->GetHandle();
	}
	mNic = nic;

	//Create the VI
	rc = VipCreateVi(nic, attribs, SendCQH, RecvCQH, &mVI);
	if (rc != VIP_SUCCESS)
	{
		throw VIException("Unable to create vi.");
	}	
}

//Queries the current VI
VIP_RETURN cVI::QueryVI()
{
	VIP_RETURN rc;

	rc = VipQueryVi(mVI, &mState, &mAttribs, &mSendQEmpty, &mRecvQEmpty);
	if (rc != VIP_SUCCESS)	
	{
		cVIError::PrintError("Unable to execute VipQueryVi command!", __LINE__, __FILE__, rc);
	}
	return rc;
}


//Accessor Functions
const VIP_VI_ATTRIBUTES* cVI::GetAttrib()
{
	if(QueryVI() != VIP_SUCCESS)
	{
		return NULL;
	}
	else
	{
		return &mAttribs; 
	}
}

VIP_VI_STATE cVI::GetState()
{
	QueryVI();
	return mState;
}

VIP_BOOLEAN	cVI::IsRecvQEmpty()
{
	QueryVI();
	return mRecvQEmpty;
}

VIP_BOOLEAN cVI::IsSendQEmpty()
{
	QueryVI();
	return mSendQEmpty;
}

//Useful Functions
VIP_RETURN	cVI::SendDone(VIP_DESCRIPTOR **DescPtr)
{
	return VipSendDone(mVI, DescPtr);
}

VIP_RETURN	cVI::RecvDone(VIP_DESCRIPTOR **DescPtr)
{
	return VipRecvDone(mVI, DescPtr);
}

VIP_RETURN	cVI::SendDone(cDesc *desc)
{

	VIP_RETURN rc;
	VIP_DESCRIPTOR *checkDesc;

	rc = VipSendDone(mVI, &checkDesc);
	if ((rc == VIP_SUCCESS) && (desc->GetDescPtr() != checkDesc)) 
	{
		cVIError::PrintError("VipSendDone called out of order and desc objects may no longer pt to correct desc handle.", __LINE__, __FILE__);
	}

	return rc;
}

VIP_RETURN	cVI::SendWait(cDesc *desc, VIP_ULONG timeout)
{
	VIP_RETURN rc;
	VIP_DESCRIPTOR *checkDesc;
	
	rc = VipSendWait(mVI, timeout, &checkDesc);
	if ((rc == VIP_SUCCESS) && (desc->GetDescPtr() != checkDesc))
	{
		cVIError::PrintError("VipSendWait called out of order and desc objects may no longer pt to correct desc handle.", __LINE__, __FILE__);
	}

	return rc;
}

VIP_RETURN  cVI::RecvDone(cDesc *desc)
{
	VIP_RETURN rc;
	VIP_DESCRIPTOR *checkDesc;
	
	rc = VipRecvDone(mVI, &checkDesc);
	if ((rc == VIP_SUCCESS) && (desc->GetDescPtr() != checkDesc))
	{
		cVIError::PrintError("RecvDone called out of order and desc objects may no longer pt to correct desc handle.", __LINE__, __FILE__);
	}

	return rc;
}

VIP_RETURN  cVI::RecvWait(cDesc *desc, VIP_ULONG timeout)
{
	VIP_RETURN rc;
	VIP_DESCRIPTOR *checkDesc;
	
	rc = VipRecvWait(mVI, timeout, &checkDesc);
	if ((rc == VIP_SUCCESS) && (desc->GetDescPtr() != checkDesc))
	{
		cVIError::PrintError("RecvWait called out of order and desc objects may no longer pt to correct desc handle.", __LINE__, __FILE__);
	}

	return rc;
}


VIP_RETURN	cVI::ConnectRequest(cVIAddr *localAddr, cVIAddr *remoteAddr, VIP_ULONG timeout)
{
	VIP_RETURN rc;

	rc = VipConnectRequest(mVI, localAddr->GetAddress(), remoteAddr->GetAddress(), timeout, &mRemoteAttribs);
	if (rc == VIP_SUCCESS)
	{
		remoteAddr->setValid(true);
	}

	return rc;
}

VIP_RETURN	cVI::ConnectWait(cVIAddr *localAddr, cVIAddr *remoteAddr, VIP_ULONG timeout)
{
	VIP_RETURN rc;

	rc = VipConnectWait(mNic, localAddr->GetAddress(), timeout, remoteAddr->GetAddress(), &mRemoteAttribs, &mConnHandle);
	if (rc == VIP_SUCCESS)
	{
		remoteAddr->setValid(VIP_TRUE);
	}

	return rc;
}

VIP_RETURN	cVI::ConnectAccept()
{
	VIP_RETURN rc;
	
	rc = VipConnectAccept(mConnHandle, mVI);
	return rc;
}

VIP_RETURN  cVI::ConnectReject()
{
	VIP_RETURN rc;
	
	rc = VipConnectReject(mConnHandle);
	return rc;
}

VIP_RETURN	cVI::Disconnect()
{
	VIP_RETURN rc;

	rc = VipDisconnect(mVI);
	return rc;
}

void cVI::Flush()
{
	VIP_RETURN 		 rc;
	VIP_DESCRIPTOR	*DescPtr;
	
	Disconnect();

	do
	{
		rc = SendDone(&DescPtr);
	} while((rc != VIP_DESCRIPTOR_ERROR) || (DescPtr != NULL));
	
	do
	{
		rc = RecvDone(&DescPtr);
	} while((rc != VIP_DESCRIPTOR_ERROR) || (DescPtr != NULL));
}

//Destructor
cVI::~cVI()
{
	VIP_RETURN 		 rc;
	VIP_DESCRIPTOR	*DescPtr;
	
	//Make sure in IDLE state.
//	if(GetState() != VIP_STATE_IDLE)
//	{
		Disconnect();
//	}
	
	do
	{
		rc = SendDone(&DescPtr);
	} while((rc != VIP_DESCRIPTOR_ERROR) || (DescPtr != NULL));
	
	do
	{
		rc = RecvDone(&DescPtr);
	} while((rc != VIP_DESCRIPTOR_ERROR) || (DescPtr != NULL));

	//Destroy the VI.
	rc = VipDestroyVi(mVI);
	if (rc != VIP_SUCCESS)
	{
		cVIError::PrintError("Unable to destroy vi.", __LINE__, __FILE__, rc);
	}
}

