/*
 * cVIError.cpp
 * 
 * Class def for VI error class. 
 *
 * Authors:  Vicky & Ted
 * Last Mod: 28 Feb 1999
 */
#include "cVIError.h"

// Global output
static ostream* gVIErrorStream = 0;

// Set the error stream.
void cVIError::SetErrorStream(ostream &outStream) 
{ 
	gVIErrorStream = &outStream; 
}

// Prints stupidity errors.
void cVIError::PrintError(const char *string, const int line, const char *file)
{
	if(!gVIErrorStream) { return; }
	(*gVIErrorStream) << "[" << file << ":" << line << "] <ERROR>: " << string << endl;	
}

// Prints the error for VIP_ERROR_CODE.
void cVIError::PrintError(const char *string, const int line, const char *file, const VIP_ERROR_CODE err)
{
	if(!gVIErrorStream) { return; }

	char errorString[128];
	
	switch(err)
	{

	case VIP_ERROR_POST_DESC:
		strcpy(errorString, "VIP_ERROR_POST_DESC");
		break;

	case VIP_ERROR_CONN_LOST:
		strcpy(errorString, "VIP_ERROR_CONN_LOST");
		break;
	case VIP_ERROR_RECVQ_EMPTY:
		strcpy(errorString, "VIP_ERROR_RECVQ_EMPTY");
		break;
	case VIP_ERROR_VI_OVERRUN:
		strcpy(errorString, "VIP_ERROR_VI_OVERRUN");
		break;

	case VIP_ERROR_RDMAW_PROT:
		strcpy(errorString, "VIP_ERROR_RDMAW_PROT");
		break;

	case VIP_ERROR_RDMAW_DATA:
		strcpy(errorString, "VIP_ERROR_RDMAW_DATA");
		break;

	case VIP_ERROR_RDMAW_ABORT:
		strcpy(errorString, "VIP_ERROR_RDMAW_ABORT");
		break;

	case VIP_ERROR_RDMAR_PROT:
		strcpy(errorString, "VIP_ERROR_RDMAR_PROT");
		break;

	case VIP_ERROR_COMP_PROT:
		strcpy(errorString, "VIP_ERROR_COMP_PROT");
		break;

	case VIP_ERROR_RDMA_TRANSPORT:
		strcpy(errorString, "VIP_ERROR_RDMA_TRANSPORT");
		break;

	case VIP_ERROR_CATASTROPHIC:
		strcpy(errorString, "VIP_ERROR_CATASTROPHIC");
		break;

	default:
		strcpy(errorString, "UNKOWN");
	}
	(*gVIErrorStream) << "[" << file << ":" << line << "] <" << errorString << ">: " << string << endl;	
}

// Prints the error for VIP_RETURN
void cVIError::PrintError(const char *string, const int line, const char *file, const VIP_RETURN rc)
{
	if(!gVIErrorStream) { return; }

	char errorString[128];
	
	switch(rc)
	{

	case VIP_SUCCESS:
		strcpy(errorString, "VIP_SUCCESS");
		break;

	case VIP_NOT_DONE:
		strcpy(errorString, "VIP_NOT_DONE");
		break;
	case VIP_INVALID_PARAMETER:
		strcpy(errorString, "VIP_INVALID_PARAMETER");
		break;
	case VIP_ERROR_RESOURCE:
		strcpy(errorString, "VIP_ERROR_RESOURCE");
		break;

	case VIP_TIMEOUT:
		strcpy(errorString, "VIP_TIMEOUT");
		break;

	case VIP_REJECT:
		strcpy(errorString, "VIP_REJECT");
		break;

	case VIP_INVALID_RELIABILITY_LEVEL:
		strcpy(errorString, "VIP_INVALID_RELIABILITY_LEVEL");
		break;

	case VIP_INVALID_MTU:
		strcpy(errorString, "VIP_INVALID_MTU");
		break;

	case VIP_INVALID_QOS:
		strcpy(errorString, "VIP_INVALID_QOS");
		break;

	case VIP_INVALID_PTAG:
		strcpy(errorString, "VIP_INVALID_PTAG");
		break;

	case VIP_INVALID_RDMAREAD:
		strcpy(errorString, "VIP_INVALID_RDMAREAD");
		break;

	case VIP_DESCRIPTOR_ERROR:
		strcpy(errorString, "VIP_DESCRIPTOR_ERROR");
		break;

	case VIP_INVALID_STATE:
		strcpy(errorString, "VIP_INVALID_STATE");
		break;

	case VIP_ERROR_NAMESERVICE:
		strcpy(errorString, "VIP_ERROR_NAMESERVICE");
		break;

	case VIP_NO_MATCH:
		strcpy(errorString, "VIP_NO_MATCH");
		break;

	case VIP_NOT_REACHABLE:
		strcpy(errorString, "VIP_NOT_REACHABLE");
		break;

	default:
		strcpy(errorString, "UNKOWN");
	}
	(*gVIErrorStream) << "[" << file << ":" << line << "] <" << errorString << ">: " << string << endl;	
	
}